## Copyright (C) 2003 Csaba Szepesvari
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


## -*- texinfo -*-
## @deftypefn {Function File} {[@var{x},@var{v}] =} impint (@var{fun},@var{dens_pdf},@var{prop_pdf},@var{prop_rnd},@var{N})
## impint approximately integrates a function using importance sampling.
## Returns the estimated integral and its estimated variance.
##
## Example:
##
## [x,v] = impint ("cos","uniform_pdf","normal5_pdf","normal5_rnd",100);
## [x,v] = impint("xnormal01_pdf","const1","normal2_pdf","normal2_rnd",1000);
## This last example should give approximately zero.
## @end deftypefn

## Author: Csaba Szepesvari
## Keywords: 

function [x,v] = impint(fun,dens_pdf,prop_pdf,prop_rnd,N)

funtype = exist (fun);
if (!(funtype==2 || funtype==3 || funtype==5))
    error ("impint: fun must be a name of a function - pls ensure the function is on LOADPATH");
endif
funtype = exist (dens_pdf);
if (!(funtype==2 || funtype==3 || funtype==5))
    error ("impint: dens_pdf must be a name of a function - pls ensure the function is on LOADPATH");
endif
funtype = exist (prop_pdf);
if (!(funtype==2 || funtype==3 || funtype==5))
    error ("impint: prop_pdf must be a name of a function - pls ensure the function is on LOADPATH");
endif
funtype = exist (prop_rnd);
if (!(funtype==2 || funtype==3 || funtype==5))
    error ("impint: prop_rnd must be a name of a function - pls ensure the function is on LOADPATH");
endif
if (!(is_scalar (N) && (N>1) && (round(N)==N)))
    error ("impint: N must be an integer larger than 1");
endif


x = feval( prop_rnd, N, 1 ); %x=sort(x); plot(x); pause;
f = feval( fun, x );         %plot(x,f); pause;
p = feval( dens_pdf, x );    %plot(x,p); pause;
q = feval( prop_pdf, x );    %plot(x,q); pause;

s = f .* p ./ q;             %plot(x,s); pause;
x = mean(s);
v = sum( (s .- x) .** 2 )/(N*(N-1));

endfunction

